
;
$NOMOD51 ; Prevents names conflict

NAME    TFPU

FP_CONV_EN EQU 1 ; Enable(1)/Disable(0) DFPMU conversions fp to int/int to fp
FP_TRIG_EN EQU 1 ; Enable(1)/Disable(0) DFPMU trigonometric sin,cos,tana,atan
FP_REGS_EN EQU 1 ; Enable(1)/Disable(0) DFPMU direct registers control

;-----------------------------------------------------------------------------
; DMA instructions definition
;-----------------------------------------------------------------------------
DMA_FPADD     EQU 28D
DMA_FPSUB     EQU 29D
DMA_FPMUL     EQU 30D
DMA_FPDIV     EQU 31D
DMA_FPSQRT    EQU 32D
DMA_FPCMP     EQU 33D
DMA_FPXAM     EQU 34D
DMA_FPCST     EQU 35D
DMA_FPIST     EQU 36D
DMA_FPLST     EQU 37D
; GAP
DMA_FPCLD     EQU 39D
DMA_FPILD     EQU 40D
DMA_FPLLD     EQU 41D
; GAP
; GAP
; GAP
DMA_FPSIN     EQU 45D
DMA_FPCOS     EQU 46D
DMA_FPTAN     EQU 47D
DMA_FPATAN    EQU 48D
DMA_FPINIT    EQU 49D
DMA_FPCLEX    EQU 50D
DMA_FPSRR     EQU 51D
DMA_FPSRW     EQU 52D
DMA_FPCRR     EQU 53D
DMA_FPCRW     EQU 54D

;-----------------------------------------------------------------------------
; DQ80251 SFR registers location
;-----------------------------------------------------------------------------
;$include (HEADER.INC)
DMAIR   DATA 0EDH
DPL     DATA 082H
DPH     DATA 083H
ACC     DATA 0E0H

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; SINGLE PRECISION
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

;-----------------------------------------------------------------------------
; '+' function replacement
;-----------------------------------------------------------------------------
?PR?FPADD?TFPU SEGMENT CODE
  PUBLIC  ?C?FPADD    ; '+'
  RSEG    ?PR?FPADD?TFPU
?C?FPADD:
  MOV     DMAIR,#DMA_FPADD
  RET

;-----------------------------------------------------------------------------
; '-' function replacement
;-----------------------------------------------------------------------------
?PR?FPSUB?TFPU SEGMENT CODE
  PUBLIC  ?C?FPSUB    ; '-'
  RSEG    ?PR?FPSUB?TFPU
?C?FPSUB:
  MOV     DMAIR,#DMA_FPSUB
  RET

;-----------------------------------------------------------------------------
; '*' function replacement
;-----------------------------------------------------------------------------
?PR?FPMUL?TFPU SEGMENT CODE
  PUBLIC  ?C?FPMUL    ; '*'
  RSEG    ?PR?FPMUL?TFPU
?C?FPMUL:
  SETB 0A0H.0
  MOV     DMAIR,#DMA_FPMUL
  CLR  0A0H.0
  RET

;-----------------------------------------------------------------------------
; '/' function replacement
;-----------------------------------------------------------------------------
?PR?FPDIV?TFPU SEGMENT CODE
  PUBLIC  ?C?FPDIV    ; '/'
  RSEG    ?PR?FPDIV?TFPU
?C?FPDIV:
  MOV     DMAIR,#DMA_FPDIV
  RET

;-----------------------------------------------------------------------------
; SQRT function replacement
;-----------------------------------------------------------------------------
?PR?SQRT?TFPU  SEGMENT CODE
  PUBLIC  sqrt?_    ; SQRT
  RSEG    ?PR?SQRT?TFPU
sqrt?_:
  MOV     DMAIR,#DMA_FPSQRT
  RET

;-----------------------------------------------------------------------------
; FPCMP function replacement
; '==', '!=', '<=', '>=', '<', '>'
;-----------------------------------------------------------------------------
?PR?FPCMP?TFPU SEGMENT CODE
  PUBLIC  ?C?FPCMP3  ; FPCMP
  RSEG    ?PR?FPCMP?TFPU
?C?FPCMP3:
  MOV     DMAIR,#DMA_FPCMP
  MOV     A,R7
  ANL     A,#0x0F
  MOV     C,ACC.0 ; A<B or Unordered
  XRL     A,#0x08
  RET

;-----------------------------------------------------------------------------
; FXAM function
;-----------------------------------------------------------------------------
?PR?FXAM?TFPU SEGMENT CODE
  PUBLIC fxam    ; _FXAM
  RSEG   ?PR?FXAM?TFPU
fxam:
  MOV   DMAIR,#DMA_FPXAM
  MOV   A,R7 ; char returns result in R11
  RET

;-----------------------------------------------------------------------------
; FCHS function replacement
;-----------------------------------------------------------------------------
?PR?FCHS?TFPU SEGMENT CODE
  PUBLIC  ?C?FPNEG  ; FCHS
  RSEG    ?PR?FCHS?TFPU
?C?FPNEG:
  MOV     A,R4
  XRL     A,#0x80
  MOV     R4,A
  RET

;-----------------------------------------------------------------------------
; FABS function replacement
;-----------------------------------------------------------------------------
?PR?FABS?TFPU SEGMENT CODE
  PUBLIC  fabs?_    ; FABS
  RSEG    ?PR?FABS?TFPU
fabs?_:
  MOV     A,R4
  ANL     A,#0x7F
  MOV     R4,A
  RET

;===============================================================================
; conditional inclusion
if (FP_TRIG_EN==1)
;-----------------------------------------------------------------------------
; SIN function replacement
;-----------------------------------------------------------------------------
?PR?sin?TFPU SEGMENT CODE
  PUBLIC  sin?_    ; sin
  RSEG    ?PR?sin?TFPU
sin?_:
  SETB 0A0H.1
  MOV     DMAIR,#DMA_FPSIN
  CLR  0A0H.1
  RET

;-----------------------------------------------------------------------------
; COS function replacement
;-----------------------------------------------------------------------------
?PR?cos?TFPU SEGMENT CODE
  PUBLIC  cos?_    ; cos
  RSEG    ?PR?cos?TFPU
cos?_:
  MOV     DMAIR,#DMA_FPCOS
  RET

;-----------------------------------------------------------------------------
; TAN function replacement
;-----------------------------------------------------------------------------
?PR?tan?TFPU SEGMENT CODE
  PUBLIC  tan?_    ; tan
  RSEG    ?PR?tan?TFPU
tan?_:
  MOV     DMAIR,#DMA_FPTAN
  RET

;-----------------------------------------------------------------------------
; ATAN function replacement
;-----------------------------------------------------------------------------
?PR?atan?TFPU SEGMENT CODE
  PUBLIC  atan?_    ; atan
  RSEG    ?PR?atan?TFPU
atan?_:
  XRL     WR2,WR2
  MOV     WR0,#0x3F80
  MOV     DMAIR,#DMA_FPATAN
  RET

endif
;===============================================================================

;===============================================================================
; conditional inclusion
if (FP_CONV_EN==1)
;-----------------------------------------------------------------------------
; CASTF function replacement FLOAT -> LONG,INT,CHAR
;-----------------------------------------------------------------------------
?PR?CASTF?TFPU  SEGMENT CODE
  PUBLIC  ?C?CASTF   ; CASTF
  RSEG    ?PR?CASTF?TFPU
?C?CASTF:
  MOV   DMAIR,#DMA_FPLST
  MOV   A,R7 ; char returns result in R11
  RET

;-----------------------------------------------------------------------------
; FCASTC function replacement CHAR -> FLOAT
;-----------------------------------------------------------------------------
?PR?FCASTC?TFPU SEGMENT CODE
  PUBLIC  ?C?FCASTC   ; FCASTC
  RSEG    ?PR?FCASTC?TFPU
?C?FCASTC:
  MOV   R7,R4 ; char passes data in R4
  MOV   DMAIR,#DMA_FPCLD
  RET

;-----------------------------------------------------------------------------
; FCASTI function replacement INT -> FLOAT
;-----------------------------------------------------------------------------
?PR?FCASTI?TFPU SEGMENT CODE
  PUBLIC ?C?FCASTI   ; FCASTI
  RSEG   ?PR?FCASTI?TFPU
?C?FCASTI:
  MOV   WR6, WR4 ; int passes data in WR4
  MOV   DMAIR,#DMA_FPILD
  RET

;-----------------------------------------------------------------------------
; FCASTL function replacement LONG -> FLOAT
;-----------------------------------------------------------------------------
?PR?FCASTL?TFPU SEGMENT CODE
  PUBLIC  ?C?FCASTL   ; FCASTL
  RSEG    ?PR?FCASTL?TFPU
?C?FCASTL:
  MOV   DMAIR,#DMA_FPLLD
  RET

endif
;===============================================================================

;===============================================================================
; conditional inclusion
if (FP_REGS_EN==1)

;-----------------------------------------------------------------------------
; FCLEX function
;-----------------------------------------------------------------------------
?PR?FCLEX?TFPU SEGMENT CODE
  PUBLIC FCLEX    ; FCLEX
  RSEG   ?PR?FCLEX?TFPU
FCLEX:
  MOV   DMAIR,#DMA_FPCLEX
  RET

;-----------------------------------------------------------------------------
; FINIT function
;-----------------------------------------------------------------------------
?PR?FINIT?TFPU SEGMENT CODE
  PUBLIC  FINIT    ; FINIT
  RSEG    ?PR?FINIT?TFPU
FINIT:
  MOV     DMAIR,#DMA_FPINIT
  RET

;-----------------------------------------------------------------------------
; FPSRR function
;-----------------------------------------------------------------------------
?PR?FPSRR?TFPU SEGMENT CODE
  PUBLIC FPSRR    ; FPSRR
  RSEG  ?PR?FPSRR?TFPU
FPSRR:
  MOV   DMAIR, #DMA_FPSRR
  MOV   A,R7 ; char passed data in ACC
  RET

;-----------------------------------------------------------------------------
; FPSRW function
;-----------------------------------------------------------------------------
?PR?FPSRW?TFPU SEGMENT CODE
  PUBLIC FPSRW   ; FPSRW
  RSEG  ?PR?FPSRW?TFPU
FPSRW:
  MOV   R7,A ; char passed data in ACC
  MOV   DMAIR, #DMA_FPSRW
  RET

;-----------------------------------------------------------------------------
; FPCRR function
;-----------------------------------------------------------------------------
?PR?FPCRR?TFPU SEGMENT CODE
  PUBLIC FPCRR    ; FPCRR
  RSEG  ?PR?FPCRR?TFPU
FPCRR:
  MOV   DMAIR, #DMA_FPCRR
  MOV   A,R7 ; char passed data in ACC
  RET

;-----------------------------------------------------------------------------
; FPCRW function
;-----------------------------------------------------------------------------
?PR?FPCRW?TFPU SEGMENT CODE
  PUBLIC FPCRW   ; FPCRW
  RSEG  ?PR?FPCRW?TFPU
FPCRW:
  MOV   R7,A ; char passed data in ACC
  MOV   DMAIR, #DMA_FPCRW
  RET

endif
;===============================================================================

END